cbuffer cbConstant
{
    float3 LightCol = float3(0.8,0.8,0.7);
    float3 AmbCol = float3(0.1,0.1,0.2);
	float3 LightDir = float3(0.71, 0, 0.71);
};

cbuffer perFrame
{
	matrix ViewProjection;
	float3 CamPos;
	float NormalMod;
}

cbuffer perDrawcall
{
	float4 OffsetParams;
}

struct VS_INPUT
{
	float4 Pos : POSITION;
	float4 Color : COLOR;
};

struct VS_OCCLUSION_INPUT
{
	float3 Pos : POSITION;
};

struct VS_OUTPUT
{
    float4 Pos : SV_POSITION;
    float3 Normal : COLOR0;
    float3 CamVec : TEXTURE0;
};

struct MRT_OUTPUT
{
    float4 Color0 : COLOR0;
    float4 Color1 : COLOR1;
};

struct VS_OCC_OUTPUT
{
    float4 Pos : SV_POSITION;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
VS_OUTPUT MandelVS( VS_INPUT input )
{
    VS_OUTPUT output = (VS_OUTPUT)0;
	
	float4 worldPos = float4(OffsetParams.xyz + input.Pos.xyz * OffsetParams.www, 1);
	output.CamVec = worldPos - CamPos;
    output.Pos = mul( worldPos, ViewProjection );
    output.Normal = input.Color.xyz * NormalMod;
    return output;
}

VS_OCC_OUTPUT OcclusionVS( VS_OCCLUSION_INPUT input )
{
    VS_OCC_OUTPUT output = (VS_OCC_OUTPUT)0;
	
    output.Pos = mul( float4(input.Pos,1), ViewProjection );
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 MandelPS( VS_OUTPUT input ) : SV_Target
{
//	return float4(input.Pos.w, (input.Pos.w*10)%1, (input.Pos.w*100)%1, 1);
	float3 normal = normalize(input.Normal);
    float lighting = dot( normal, LightDir ) *0.5+0.5;
    float3 matColor = normal*0.2+0.8;
    float3 camVec = normalize(input.CamVec);
	float3 reflectionDir = camVec - normal * 2 * dot(camVec, normal);
	float specular = pow(saturate(dot(reflectionDir, LightDir)), 128); 
	float3 finalColor = matColor * (LightCol * lighting + AmbCol);// +specular * 0.5;
    return float4(finalColor,1);
}

MRT_OUTPUT MandelMrtPS( VS_OUTPUT input ) : SV_Target
{
	//if((input.CamVec + CamPos).z > 0)
	//	discard;
	float3 normal = normalize(input.Normal);
    float lighting = dot( normal, LightDir ) *0.5+0.5;
    float3 matColor = normal*0.2+0.8;
    float3 camVec = normalize(input.CamVec);
	float3 reflectionDir = camVec - normal * 2 * dot(camVec, normal);
	float specular = pow(saturate(dot(reflectionDir, LightDir)), 128); 
    float3 finalColor = matColor * (LightCol * lighting + AmbCol) + specular * 0.5;
	MRT_OUTPUT output;
    output.Color0 = float4(finalColor,1);
	output.Color1 = float4(input.Pos.w, 0,0,0);
	return output;
}

float4 OcclusionPS( VS_OCC_OUTPUT input ) : SV_Target
{
    return float4(1,0,0,1);
}

//--------------------------------------------------------------------------------------

RasterizerState rsWireframe 
{ 
	FillMode = WireFrame; 
	CullMode = None;
};
RasterizerState rsNoCull 
{ 
	CullMode = None;
	MultiSampleEnable = True;
};
RasterizerState rsBackCull 
{ 
	CullMode = Back;
	MultiSampleEnable = True;
};
RasterizerState rsFrontCull 
{ 
	CullMode = Front;
	MultiSampleEnable = True;
};
DepthStencilState dssStandard 
{
};
DepthStencilState DisableDepthWrite
{
    DepthEnable = TRUE;
    DepthWriteMask = ZERO;
};
BlendState OccTestBlendState
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = FALSE;
    RenderTargetWriteMask[0] = 0x0;
};
BlendState bsStandard
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = FALSE;
    RenderTargetWriteMask[0] = 0x0f;
};

//--------------------------------------------------------------------------------------

technique11 MandRender
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsStandard, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, MandelVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, MandelPS() ) );
    }
    pass P1
    {
		SetRasterizerState(rsBackCull);
    }
    pass P2
    {
		SetRasterizerState(rsFrontCull);
    }
}
technique11 MandRenderMrt
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsStandard, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, MandelVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, MandelMrtPS() ) );
    }
}

technique11 OcclusionRender
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(DisableDepthWrite, 0);
		SetBlendState(OccTestBlendState, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, OcclusionVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, OcclusionPS() ) );
    }
}

technique11 DebugRender
{
    pass P0
    {
		SetRasterizerState(rsWireframe);
		SetDepthStencilState(DisableDepthWrite, 0);
		SetBlendState(bsStandard, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, OcclusionVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, OcclusionPS() ) );
    }
}

