cbuffer cbConstant
{
    float3 LightCol = float3(0.8,0.8,0.7);
};

cbuffer perFrame
{
	float fogDensity;
}

cbuffer cbTexOffsets
{
    float4 texOffsets[8];
    float4 filterTexOffsets[8];
};

Texture2D texDepth;
Texture2D texDepthDownsampled;
Texture2D texDepthDownsampled2;
Texture2D texSSAO;

SamplerState sampleLinearClamp
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct VS_INPUT
{
	float3 Pos : POSITION;
	float3 TexCoord : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 Pos : SV_POSITION;
    float3 TexCoord : TEXTURE0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
VS_OUTPUT StandardVS( VS_INPUT input )
{
    VS_OUTPUT output = (VS_OUTPUT)0;
	output.Pos = float4(input.Pos,1);//*float4(0.5,0.5,1,1);
    output.TexCoord = input.TexCoord;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 DownsamplePS( VS_OUTPUT input ) : SV_Target
{
	float3 coord = input.Pos.xyz*float3(2,2,1);
	float depthSample = texDepth.Load( coord ).x;
	depthSample += texDepth.Load( coord + float3(1,0,0) ).x;
	depthSample += texDepth.Load( coord + float3(1,1,0) ).x;
	depthSample += texDepth.Load( coord + float3(0,1,0) ).x;
    return float4(depthSample*0.25,0,0,1);
}

float4 SSAOPS( VS_OUTPUT input ) : SV_Target
{
	float3 coord = input.Pos.xyz;
	float depth = texDepth.Load( coord ).x;
	
	int baseIndex = ((int(input.Pos.x)&1) + (int(input.Pos.y)&1) * 2) * 2; 
	float col = 1;
	for(int b=0;b<2;++b)
	{
		for(int a=0;a<4;++a)
		{
			float dist = texOffsets[baseIndex+b].z + a*8;
			float2 offset = texOffsets[baseIndex+b].xy * dist * texOffsets[baseIndex+b].w;
			float depth1 = texDepth.Load( coord + float3(offset,0)).x;
			float depth2 = texDepth.Load( coord + float3(-offset,0)).x;

			float dif = ((depth1+depth2)*0.5-depth)/depth;
			dif *= saturate(1+dif*10);
			col *= saturate(1+dif/max(10,dist)*100);
		}
	}
    return float4(1-col,0,0,1);
}

float4 ApplyPS( VS_OUTPUT input ) : SV_Target
{
	float3 coord = input.Pos.xyz;
	float ssao = texSSAO.Load( coord ).x;
	float depth = texDepth.Load( coord ).x;
	float total = 1;
	
	for(int a=0;a<8;++a)
	{
		float3 offset = float3(filterTexOffsets[a].xy, 0);
		float ssao1 = texSSAO.Load( coord+offset ).x;
		float depth1 = texDepth.Load( coord+offset ).x;
		float use = filterTexOffsets[a].z * saturate(1-abs(depth-depth1)/depth*25);
		ssao += ssao1 * use;
		total += use;
	}
	float shade = (ssao/total)*0.8f;
	float fog = 1-pow(fogDensity, depth);
	float alpha = 1-(1-fog)*(1-shade);
	const float3 fogColor = float3(0.05f, 0.125f, 0.3f);
	float3 color = fogColor * fog;
    return float4(color,alpha);
}

float4 CopyPS(VS_OUTPUT input) : SV_Target
{
	float3 coord = input.Pos.xyz;
	float3 color = texSSAO.Sample(sampleLinearClamp, input.TexCoord.xy);
	return float4(color, 1);
}

//--------------------------------------------------------------------------------------

RasterizerState rsNoCull 
{ 
	CullMode = None;
	DepthClipEnable = FALSE;
};
DepthStencilState dssStandard 
{
    DepthEnable = FALSE;
    DepthWriteMask = ZERO;
    StencilEnable = FALSE;
};
BlendState bsStandard
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = FALSE;
    RenderTargetWriteMask[0] = 0x0f;
};
BlendState bsAlphaBlend
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    RenderTargetWriteMask[0] = 0x0f;
};

//--------------------------------------------------------------------------------------

technique11 DownsampleRender
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsStandard, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, StandardVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, DownsamplePS() ) );
    }
}
technique11 SSAORender
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsStandard, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, StandardVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, SSAOPS() ) );
    }
}
technique11 ApplyPostfxRender
{
    pass P0
    {
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsAlphaBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetVertexShader( CompileShader( vs_5_0, StandardVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_5_0, ApplyPS() ) );
    }
}
technique11 CopyRender
{
	pass P0
	{
		SetRasterizerState(rsNoCull);
		SetDepthStencilState(dssStandard, 0);
		SetBlendState(bsAlphaBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
		SetVertexShader(CompileShader(vs_5_0, StandardVS()));
		SetGeometryShader(NULL);
		SetPixelShader(CompileShader(ps_5_0, CopyPS()));
	}
}
